@@ -9,29 +9,10 @@ include Types
 include Exported_types
 module Version_error = Versioned.Version_error
 module Decl = Decl
+module Sub = Sub
 
 module type Fiber = Fiber_intf.S
 
-module Sub = struct
-  type 'a t =
-    { poll : (Id.t, 'a option) Decl.Request.witness
-    ; cancel : Id.t Decl.Notification.witness
-    ; id : Procedures.Poll.Name.t
-    }
-
-  let of_procedure p =
-    let open Procedures.Poll in
-    { poll = (poll p).decl; cancel = (cancel p).decl; id = name p }
-
-  let poll t = t.poll
-
-  let poll_cancel t = t.cancel
-
-  module Id = Procedures.Poll.Name
-
-  let id t = t.id
-end
-
 module Public = struct
   module Request = struct
     type ('a, 'b) t = ('a, 'b) Decl.Request.witness
@@ -115,9 +96,9 @@ module Client = struct
       ?id:Id.t -> t -> 'a Sub.t -> ('a Stream.t, Version_error.t) result fiber
 
     module Batch : sig
-      type t
+      type client := t
 
-      type client
+      type t
 
       val create : client -> t
 
@@ -132,7 +113,6 @@ module Client = struct
 
       val submit : t -> unit fiber
     end
-    with type client := t
 
     module Handler : sig
       type t
@@ -148,6 +128,7 @@ module Client = struct
       | Request : ('a, 'b) Decl.request -> proc
       | Notification : 'a Decl.notification -> proc
       | Poll : 'a Procedures.Poll.t -> proc
+      | Handle_request : ('a, 'b) Decl.request * ('a -> 'b fiber) -> proc
 
     val connect_with_menu :
          ?handler:Handler.t
@@ -165,42 +146,14 @@ module Client = struct
       -> 'a fiber
   end
 
-  module Make (Fiber : sig
-    type 'a t
+  module Make
+      (Fiber : Fiber_intf.S) (Chan : sig
+        type t
 
-    val return : 'a -> 'a t
+        val write : t -> Sexp.t list option -> unit Fiber.t
 
-    val fork_and_join_unit : (unit -> unit t) -> (unit -> 'a t) -> 'a t
-
-    val parallel_iter : (unit -> 'a option t) -> f:('a -> unit t) -> unit t
-
-    val finalize : (unit -> 'a t) -> finally:(unit -> unit t) -> 'a t
-
-    module O : sig
-      val ( let* ) : 'a t -> ('a -> 'b t) -> 'b t
-
-      val ( let+ ) : 'a t -> ('a -> 'b) -> 'b t
-    end
-
-    module Ivar : sig
-      type 'a fiber
-
-      type 'a t
-
-      val create : unit -> 'a t
-
-      val read : 'a t -> 'a fiber
-
-      val fill : 'a t -> 'a -> unit fiber
-    end
-    with type 'a fiber := 'a t
-  end) (Chan : sig
-    type t
-
-    val write : t -> Sexp.t list option -> unit Fiber.t
-
-    val read : t -> Sexp.t option Fiber.t
-  end) =
+        val read : t -> Sexp.t option Fiber.t
+      end) =
   struct
     open Fiber.O
     module V = Versioned.Make (Fiber)
@@ -319,20 +272,15 @@ module Client = struct
     let terminate_with_error t message info =
       Fiber.fork_and_join_unit
         (fun () -> terminate t)
-        (fun () -> Code_error.raise message info)
+        (fun () ->
+          (* TODO stop using code error here. If [terminate_with_error] is
+             called, it's because the other side is doing something unexpected,
+             not because we have a bug *)
+          Code_error.raise message info)
 
-    let send conn (packet : Packet.Query.t list option) =
+    let send conn (packet : Packet.t list option) =
       let sexps =
-        Option.map packet
-          ~f:
-            (List.map ~f:(function
-              | Packet.Query.Notification p ->
-                Conv.to_sexp (Conv.record Call.fields) p
-              | Request (id, request) ->
-                let conv =
-                  Conv.record (Conv.both Id.required_field Call.fields)
-                in
-                Conv.to_sexp conv (id, request)))
+        Option.map packet ~f:(List.map ~f:(Conv.to_sexp Packet.sexp))
       in
       Chan.write conn.chan sexps
 
@@ -432,10 +380,7 @@ module Client = struct
       | true -> k call
       | false ->
         let err =
-          let payload =
-            Sexp.record
-              [ ("method", Atom call.method_); ("params", call.params) ]
-          in
+          let payload = Conv.to_sexp (Conv.record Call.fields) call in
           Response.Error.create ~payload
             ~message:"notification sent while connection is dead"
             ~kind:Code_error ()
@@ -545,7 +490,7 @@ module Client = struct
     module Batch = struct
       type nonrec t =
         { client : t
-        ; mutable pending : Packet.Query.t list
+        ; mutable pending : Packet.t list
         }
 
       let create client = { client; pending = [] }
@@ -564,7 +509,7 @@ module Client = struct
         match ivar with
         | Error e -> Fiber.return (Error e)
         | Ok ivar ->
-          t.pending <- Packet.Query.Request (id, call) :: t.pending;
+          t.pending <- Packet.Request (id, call) :: t.pending;
           let* res = Fiber.Ivar.read ivar in
           (* currently impossible because there's no batching for polling and
              cancellation is only available for polled requests *)
@@ -581,7 +526,7 @@ module Client = struct
     let read_packets t packets =
       let* () =
         Fiber.parallel_iter packets ~f:(function
-          | Packet.Reply.Notification n -> (
+          | Packet.Notification n -> (
             if
               String.equal n.method_ Procedures.Server_side.abort.decl.method_
               && not t.handler_initialized
@@ -591,10 +536,12 @@ module Client = struct
                   n.params
               with
               | Ok msg -> t.on_preemptive_abort msg
-              | Error _ ->
-                Code_error.raise
+              | Error error ->
+                terminate_with_error t
                   "fatal: server aborted connection, but couldn't parse reason"
-                  [ ("reason", Sexp.to_dyn n.params) ]
+                  [ ("reason", Sexp.to_dyn n.params)
+                  ; ("error", Conv.dyn_of_error error)
+                  ]
             else
               let* handler = t.handler in
               let* result = V.Handler.handle_notification handler () n in
@@ -605,6 +552,10 @@ module Client = struct
                   ; ("notification", Call.to_dyn n)
                   ]
               | Ok () -> Fiber.return ())
+          | Request (id, req) ->
+            let* handler = t.handler in
+            let* result = V.Handler.handle_request handler () (id, req) in
+            send t (Some [ Response (id, result) ])
           | Response (id, response) -> (
             match Table.find t.requests id with
             | Some status -> (
@@ -655,8 +606,9 @@ module Client = struct
       | Request : ('a, 'b) Decl.request -> proc
       | Notification : 'a Decl.notification -> proc
       | Poll : 'a Procedures.Poll.t -> proc
+      | Handle_request : ('a, 'b) Decl.request * ('a -> 'b Fiber.t) -> proc
 
-    let setup_versioning ?(private_menu = []) ~(handler : Handler.t) () =
+    let setup_versioning ~private_menu ~(handler : Handler.t) =
       let module Builder = V.Builder in
       let t : unit Builder.t = Builder.create () in
       (* CR-soon cwong: It is a *huge* footgun that you have to remember to
@@ -680,6 +632,7 @@ module Client = struct
       Builder.declare_notification t Procedures.Poll.(cancel diagnostic);
       Builder.declare_notification t Procedures.Poll.(cancel progress);
       List.iter private_menu ~f:(function
+        | Handle_request (r, h) -> Builder.implement_request t r (fun () -> h)
         | Request r -> Builder.declare_request t r
         | Notification n -> Builder.declare_notification t n
         | Poll p ->
@@ -693,17 +646,18 @@ module Client = struct
         let+ read = Chan.read chan in
         Option.map read ~f:(fun sexp ->
             match
-              Conv.of_sexp Packet.Reply.sexp ~version:initialize.dune_version
-                sexp
+              Conv.of_sexp Packet.sexp ~version:initialize.dune_version sexp
             with
             | Error e -> raise (Abort (Invalid_session e))
             | Ok message -> message)
       in
-      let builder = setup_versioning ~handler ~private_menu () in
-      let on_preemptive_abort = handler.abort in
+      let builder = setup_versioning ~handler ~private_menu in
       let handler_var = Fiber.Ivar.create () in
-      let handler = Fiber.Ivar.read handler_var in
-      let client = create ~initialize ~chan ~handler ~on_preemptive_abort in
+      let client =
+        let on_preemptive_abort = handler.abort in
+        let handler = Fiber.Ivar.read handler_var in
+        create ~initialize ~chan ~handler ~on_preemptive_abort
+      in
       let run () =
         let* init =
           let id = Id.make (List [ Atom "initialize" ]) in
@@ -729,7 +683,7 @@ module Client = struct
                 in
                 Version_negotiation.Request.to_call request
               in
-              let+ resp = request_untyped client (id, supported_versions) in
+              let* resp = request_untyped client (id, supported_versions) in
               (* we don't allow cancelling negotiation *)
               match no_cancel_raise_connection_dead id resp with
               | Error e -> raise (Response.Error.E e)
@@ -741,9 +695,9 @@ module Client = struct
                 | Error e -> raise (Abort (Invalid_session e))
                 | Ok (Selected methods) -> (
                   match Menu.of_list methods with
-                  | Ok m -> m
+                  | Ok m -> Fiber.return m
                   | Error (method_, a, b) ->
-                    Code_error.raise
+                    terminate_with_error client
                       "server responded with invalid version menu"
                       [ ( "duplicated"
                         , Dyn.Tuple [ Dyn.String method_; Dyn.Int a; Dyn.Int b ]
